@@ -2,12 +2,13 @@ package Tie::Hash::Attribute;
 use 5.006;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.04';
-
+our $VERSION = '0.05';
 our @ISA = 'Tie::Hash';
 
+use HTML::Entities;
+
 sub TIEHASH     { bless {@_[1..@_-1]}, $_[0] }
-sub STORE       { $_[0]{$_[1]} = $_[2] }
+sub STORE       { $_[0]{$_[1]}=$_[2] }
 sub SCALAR      { _mk_str($_[0]) }
 sub EXISTS      { exists $_[0]{$_[1]} }
 sub FIRSTKEY    { each %{$_[0]} }
@@ -16,7 +17,8 @@ sub DELETE      { delete $_[0]{$_[1]} }
 sub CLEAR       { delete $_[0]{$_} for keys %{$_[0]} }
 
 sub FETCH {
-    my ($self,$arg) = @_;
+    my $self = shift;
+    my $arg  = shift;
     return $self->{$arg} unless substr($arg,0,1) eq '-';
     $arg =~ s/^-//;
     return _mk_str( $self->{$arg} );
@@ -25,15 +27,29 @@ sub FETCH {
 sub _mk_str {
     my $hash = shift;
     my $str = '';
+    my %seen;
     for my $key (sort keys %$hash) {
+        next if $seen{lc$key}++;
         my $val = defined($hash->{$key}) ? $hash->{$key} : '';
         $val  = _stringify( $val )  if ref $val eq 'HASH';
         $val  = _rotate( $val )     if ref $val eq 'ARRAY';
-        $str .= qq( $key="$val")    unless $val =~ /^$/;
+        $str .= sprintf ' %s="%s"', _key( $key ), _val( $val );
     }
     return $str;
 }
 
+sub _key {
+    my $key = shift;
+    return '' unless defined $key;
+    $key = encode_entities( $key, q("'>/=) );
+    return encode_entities( $key, '^\n\x20-\x25\x27-\x7e' );
+}
+sub _val {
+    my $val = shift;
+    $val = '' if $val =~ /^\s+$/;
+    return encode_entities( $val, '"' );
+}
+
 sub _rotate {
     my $ref  = shift;
     my $next = shift @$ref;
@@ -121,17 +137,19 @@ a dash prepended to it:
 Or access the entire hash as a scalar:
 
   print scalar %hash;
+    # foo="1" bar="2" baz="3"
 
 You can use this to to aide in the creation of HTML tags:
 
   print '<table>';
   for my $row (@rows) {
-      printf '<tr%s>', scalar %tr;
+      printf '<tr%s>', scalar %tr_tag;
       for my $col (@$row) {
-          printf '<td%s>%s</td>', scalar %td, $col;
+          printf '<td%s>%s</td>', scalar %td_tag, $col;
       }
       print '</tr>';
   }
+  print '/<table>';
 
 The decision on which style to apply to a row is deferred to
 the tied hash.  Just assign an array reference to the key and
@@ -139,6 +157,41 @@ each value will be rotated.
 
   %tr_tag = ( class => [qw( odd even )] );
 
+All HTML attribute strings returned will have a leading space.
+This prevents the client from having to check for the potential
+of no attributes, which would leave tags with trailing space
+inside the start tag: <foo ></foo>
+
+=head1 REQUIRES
+
+=over 4
+
+=item * L<HTML::Entities>
+
+Used to ensure key names and values do not break certain HTML5 Syntax rules:
+
+=over 8
+
+=item * no case-insenstive matches for attribute names in start tag
+
+=item * keys: HTML encode control chars and the following: " ' > / =
+
+=item * values: HTML encode the " char
+
+=back 
+
+As such all attributes values are enclosed in double quotes, not single quotes.
+
+=back
+
+=head1 SEE ALSO
+
+=over 4
+
+=item * L<http://www.w3.org/TR/html5/syntax.html#attributes-0>
+
+=back
+
 =head1 BUGS AND LIMITATIONS
 
 Assignment stops at the second nested key, which will use the
@@ -152,23 +205,13 @@ third as its value:
 This is an intended limitation. If there are other keys in the
 second nested hash, then the first key in alphabetical order wins.
 
-There are other limitations that will be corrected over a short time:
-
-=over 4
-
-=item * Need to apply correct encoding to keys.
-
-=item * Need to detect when " is in a value.
-
-=back
-
 Please report any bugs or feature requests to either
 
 =over 4
 
-=item * Email: C<bug-spreadsheet-html at rt.cpan.org>
+=item * Email: C<bug-tie-hash-attribute at rt.cpan.org>
 
-=item * Web: L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Spreadsheet-HTML>
+=item * Web: L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Tie-Hash-Attribute>
 
 =back
 
@@ -196,14 +239,6 @@ You can also look for information at:
 
 =back
 
-=head1 SEE ALSO
-
-=over 4
-
-=item * L<http://www.w3.org/TR/html5/syntax.html#attributes-0>
-
-=back
-
 =head1 AUTHOR
 
 Jeff Anderson, C<< <jeffa at cpan.org> >>
